/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */

package com.keyware.sonar.java.rules.checkers;

import org.sonar.check.Rule;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;
import org.sonar.plugins.java.api.tree.*;

import java.util.Collections;
import java.util.List;

/**
 * 通过用户名口令、数据证书等其他手段对主机身份进行鉴别
 *
 * @author RenFengJiang
 * @date 2024/7/3
 */
@Rule(key = "HostIdentityChecker")
public class HostIdentityChecker extends IssuableSubscriptionVisitor {

    @Override
    public List<Tree.Kind> nodesToVisit() {
        return Collections.singletonList(Tree.Kind.METHOD);
    }
    @Override
    public void visitNode(Tree tree) {
        MethodTree methodTree = (MethodTree) tree;
        List<VariableTree> parameters = methodTree.parameters();
//        盘带是否是文件上传类
        boolean boo = parameters.stream().anyMatch(type -> "HttpServletRequest".equals(type.type().toString()));
        if(boo){
            NodeIf nodeIf = new NodeIf();
            methodTree.block().accept(nodeIf);
            if(nodeIf.getNameBoolean() || nodeIf.getPasswordBoolean()){
                context.reportIssue(this,methodTree.simpleName(),"通过用户名口令、数据证书等其他手段对主机身份进行鉴别");
            }
        }

    }

    public class NodeIf extends BaseTreeVisitor {

        public boolean nameBoolean = true;
        private boolean passwordBoolean = true;
        boolean getNameBoolean(){
            return nameBoolean;
        }
        boolean getPasswordBoolean(){
            return passwordBoolean;
        }
        @Override
        public void visitIfStatement(IfStatementTree tree) {
            //获取到if表达式
            ExpressionTree condition = tree.condition();
            if (condition != null && condition instanceof BinaryExpressionTree) {
                BinaryExpressionTree binaryExpressionTree = (BinaryExpressionTree) condition;
                //判断是否进行if判断
                if ("username".equals(binaryExpressionTree.leftOperand().toString())) {
                    nameBoolean = false;
                } else if ("password".equals(binaryExpressionTree.rightOperand().toString())) {
                    passwordBoolean = false;
                }
            }
            if (condition != null && condition instanceof IdentifierTree) {
                IdentifierTree identifierTree = (IdentifierTree) condition;
                //判断是否进行if判断
                if ("username".equals(identifierTree.name())) {
                    nameBoolean = false;
                } else if ("password".equals(identifierTree.name())) {
                    passwordBoolean = false;
                }
            }
        }
    }
}
