/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：信息安全性设计准则检查插件
 * 项目描述：用于检查源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */

package com.keyware.sonar.java.rules.checkers;

import org.sonar.check.Rule;
import org.sonar.plugins.java.api.IssuableSubscriptionVisitor;
import org.sonar.plugins.java.api.tree.*;

import java.util.Collections;
import java.util.List;

/**
 * 用户输入口令时应对口令域进行掩饰。用户输入的每一个字符都应该以星号形式回显。
 *
 * @author RenFengJiang
 * @date 2024/7/3
 */
@Rule(key = "PasswordInputTagJavaChecker")
public class PasswordInputTagJavaChecker extends IssuableSubscriptionVisitor {
    @Override
    public List<Tree.Kind> nodesToVisit() {
        /**
         * Tree.Kind.METHOD：方法节点
         * Tree.Kind.BLOCK：方法的代码块节点
         * Tree.Kind.METHOD_INVOCATION： 方法的调用节点
         */
        return Collections.singletonList(Tree.Kind.BLOCK);
    }

    @Override
    public void visitNode(Tree tree) {
        BlockTree node = (BlockTree) tree;
        MethodeBodyVisitor methodeBodyVisitor = new MethodeBodyVisitor(this, node);
        node.accept(methodeBodyVisitor);
    }

    static class MethodeBodyVisitor extends BaseTreeVisitor {
        private BlockTree blockTree;
        private PasswordInputTagJavaChecker checker;
        public MethodeBodyVisitor(PasswordInputTagJavaChecker checker, BlockTree blockTree){
            this.checker = checker;
            this.blockTree = blockTree;
        }

        @Override
        public void visitNewClass(NewClassTree tree) {
            String name = tree.identifier().toString();
            if("JTextField".equals(name)){
                MethodeCall methodeCall = new MethodeCall();
                blockTree.accept(methodeCall);
                if(methodeCall.getEchoBoolean()){
                    checker.context.reportIssue(checker, tree, "Java用户输入口令时应对口令域进行掩饰,用户输入的每一个字符都应该以星号形式回显");
                }
            }

        }
    }

    static class MethodeCall extends BaseTreeVisitor {
        //是否存在JTextField输入
        private boolean echoBoolean = true;

        public boolean getEchoBoolean() {
            return echoBoolean;
        }

        @Override
        public void visitMethodInvocation(MethodInvocationTree tree) {
            //获取到方法调用
            ExpressionTree expressionTree = tree.methodSelect();
            if (expressionTree.is(Tree.Kind.MEMBER_SELECT)) {
                MemberSelectExpressionTree memberSelectExpressionTree = (MemberSelectExpressionTree) expressionTree;
                if ("setEchoChar".equals(memberSelectExpressionTree.identifier().name())) {
                    echoBoolean = false;
                }
            }
        }
    }

}
